package xyz.veronie.bgg.ui.parts;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.result.BggApi;
import xyz.veronie.bgg.result.ResultConfig;
import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.result.Thing;
import xyz.veronie.bgg.result.ThingProvider;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.ResultAction;
import xyz.veronie.bgg.types.SourceFilter;
import xyz.veronie.bgg.ui.filters.AgeSourceFilter;
import xyz.veronie.bgg.ui.filters.BggUserSourceFilter;
import xyz.veronie.bgg.ui.filters.FamilySourceFilter;
import xyz.veronie.bgg.ui.filters.GeeklistSourceFilter;
import xyz.veronie.bgg.ui.filters.RankSourceFilter;
import xyz.veronie.bgg.ui.filters.SearchSourceFilter;
import xyz.veronie.bgg.ui.filters.YearSourceFilter;
import xyz.veronie.bgg.ui.helpers.CommonControls;



/// Has controls to download a list of thing IDs (+some meta info) from BGG.
/// There are different ways to configure which IDs to retrieve.
public class PreparePart {

	@Inject 
	EPartService partService;

	@Inject
	private ResultConfigManager configManager;
	
	@Inject 
	private IEventBroker eventBroker;
	
	@Inject 
	private ThingProvider thingProvider;
	
	// inject all source filter composites
	@Inject private BggUserSourceFilter bggUserSourceFilter;
	@Inject private GeeklistSourceFilter geeklistSourceFilter;
	@Inject private FamilySourceFilter familySourceFilter;
	@Inject private AgeSourceFilter ageSourceFilter;
	@Inject private RankSourceFilter rankSourceFilter;
	@Inject private YearSourceFilter yearSourceFilter;
	@Inject private SearchSourceFilter searchSourceFilter;
	@Inject private BggApi bggApi;

	private Composite mainComposite;
	
	@PostConstruct
	public void createControls(Composite parent) {
		mainComposite = new Composite(parent, SWT.FILL);
		mainComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		mainComposite.setLayout(new GridLayout(1, false));
		
//		// contains configuration options for download of the bgg thing ID list
//		Group dlConfigGroup = new Group(main, SWT.SHADOW_ETCHED_IN);
//		GridData configGrid = new GridData(SWT.FILL, SWT.FILL, true, true);
//		dlConfigGroup.setText("Configure filters...");
//		dlConfigGroup.setLayoutData(configGrid);
//		GridLayout layoutG = new GridLayout(2, false);
//		dlConfigGroup.setLayout(layoutG);
		
		Composite sourceSelectCompo = new Composite(mainComposite, SWT.FILL);
		sourceSelectCompo.setLayout(new GridLayout(2, false));
		
		Label lblSource = new Label(sourceSelectCompo, SWT.LEFT);
		lblSource.setText("Source: ");
		lblSource.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		
		// choose the bgg data source
		ComboViewer cbSource = new ComboViewer(sourceSelectCompo, SWT.READ_ONLY);
		cbSource.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		cbSource.setContentProvider(ArrayContentProvider.getInstance());
		List<SourceFilter> sources = new ArrayList<SourceFilter>();
		for(SourceFilter sf : SourceFilter.values()) {
			sources.add(sf);
		}
		cbSource.setInput(sources);
		// TODO: implement all the sources
		cbSource.setSelection(new StructuredSelection(configManager.getResultConfig().source));
		// listener is configured further below

		CommonControls.makeEmptySpace(mainComposite, 2);
		

        // area to display filter controls 

		Group gFilters = new Group(mainComposite, SWT.SHADOW_ETCHED_IN);
        gFilters.setText("");
        gFilters.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        
		showFilter(gFilters, (IStructuredSelection)cbSource.getSelection()); 		// set content to initial selection
		cbSource.addSelectionChangedListener(new ISelectionChangedListener() {      // later, its content will be set via the dropdown
		    @Override
		    public void selectionChanged(SelectionChangedEvent event) {
		        IStructuredSelection selection = (IStructuredSelection) event.getSelection();
	        	// ASK: this is strange to me. Why is filterComposite known INSIDE a free function?
		        showFilter(gFilters, selection);
		        eventBroker.send(EventConstants.TOPIC_SOURCE_CHANGED, selection.getFirstElement());
		    }

		});
		

		// next row
		
		Composite fetchCompo = new Composite(mainComposite, SWT.FILL);
		fetchCompo.setLayout(new GridLayout(4, false));
		
		// choose action on result list
		Button btDownload = new Button(fetchCompo, SWT.PUSH);
		GridData gd = new GridData(SWT.LEFT, SWT.FILL, false, false);
		btDownload.setLayoutData(gd);
		btDownload.setText("Fetch");
		btDownload.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
		        ResultConfig resultConfig = configManager.getResultConfig();
		        
		        IStructuredSelection selection = (IStructuredSelection) cbSource.getSelection();
		        if(selection.size() == 0) return;

		        if(selection.getFirstElement() == SourceFilter.BGG_USER) {
		        	String user = resultConfig.user;
		        	if(user == null || user.isEmpty()) {
		        		MessageDialog.openError(mainComposite.getShell(), "", "Please enter a user name.");
		        		return;
		        	} else {
				        eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching " + selection.getFirstElement().toString() + " '" + user + "'...");
				        try {
				        	ArrayList<Thing> things = bggApi.getThingsForUser(user);
				        	useThingsOnResult(things);
				        }
				        catch(IllegalArgumentException ex) {
				        	MessageDialog.openError(mainComposite.getShell(), "", ex.getMessage());
				        }
		        	}
		        	
		        } else if(selection.getFirstElement() == SourceFilter.GEEKLIST) {
		        	Integer geeklistId = resultConfig.geeklistId;
		        	if(geeklistId == null) {
		        		MessageDialog.openError(mainComposite.getShell(), "", "Please enter a geeklist id.");
		        		return;
		        	} else {
		        		eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching for geeklist id '" + geeklistId + "'");
		        		try {
		        			ArrayList<Thing> things = bggApi.getThingsForGeeklist(geeklistId);
		        			useThingsOnResult(things);
		        		}
				        catch(IllegalArgumentException ex) {
				        	MessageDialog.openError(mainComposite.getShell(), "", ex.getMessage());
				        }
		        	}
		        } else if(selection.getFirstElement() == SourceFilter.FAMILY) {
		        	Integer familyId = resultConfig.familyId;
		        	if(familyId == null) {
		        		MessageDialog.openError(mainComposite.getShell(), "", "Please enter a family id.");
		        		return;
		        	} else {
		        		eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching for family id '" + familyId + "'");
		        		try {
		        			ArrayList<Thing> things = bggApi.getThingsForFamily(familyId);
		        			useThingsOnResult(things);
		        		}
				        catch(IllegalArgumentException ex) {
				        	MessageDialog.openError(mainComposite.getShell(), "", ex.getMessage());
				        }
		        	}
		        }
		    }

			private void useThingsOnResult(ArrayList<Thing> things) {
				switch(configManager.getResultConfig().action) {
				case REP:
					thingProvider.replaceThings(things);
					break;
				case ADD:
					thingProvider.addThings(things);
					break;
				case SUB:
					thingProvider.subtractThings(things);
					break;
				case AND:
					thingProvider.intersectThings(things);
					break;
				case ONLY_NEW:
					thingProvider.keepOnlyNew(things);
					break;
				}
				eventBroker.send(EventConstants.TOPIC_RESULT_CHANGED, "");
				eventBroker.send(EventConstants.TOPIC_STATUS, "Fetched " + Integer.toString(things.size()) + " things.");
			}
		});

		
		Label lblAct = new Label(fetchCompo, SWT.LEFT);
		lblAct.setText("and");
		lblAct.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false, false));
		
		ComboViewer cbAct = new ComboViewer(fetchCompo, SWT.READ_ONLY);
		GridData gdact = new GridData(SWT.FILL, SWT.FILL, false, false);
		cbAct.getCombo().setLayoutData(gdact);
		cbAct.setContentProvider(ArrayContentProvider.getInstance());
		List<ResultAction> actions = new ArrayList<ResultAction>();
		for(ResultAction act : ResultAction.values()) {
			actions.add(act);
		}
		cbAct.setInput(actions);
		cbAct.setSelection(new StructuredSelection(configManager.getResultConfig().action));
		cbAct.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_ACTION_CHANGED, selection.getFirstElement());
			}
		});
		
		
		Label toResultLabel = new Label(fetchCompo, SWT.LEFT);
		toResultLabel.setText("result");
		toResultLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));        

		mainComposite.pack();
		mainComposite.layout();
		
	}
	
	
	/// show different filter controls depending on selection in cbSource ComboViewer
	private void showFilter(Composite parent, IStructuredSelection selection) {	
		SourceFilter elem = (SourceFilter) selection.getFirstElement();

		// clean up
		for(Control child : parent.getChildren()) {
			child.dispose();
		}
		
		// create a new filter area based on selection:
		switch(elem) {
		case BGG_USER:
			System.out.println("construct " + elem);
			bggUserSourceFilter.create(parent, SWT.FILL);
			break;
		case GEEKLIST:
			System.out.println("construct " + elem);
			geeklistSourceFilter.create(parent, SWT.FILL);
			break;
		case FAMILY:
			System.out.println("construct " + elem);
			familySourceFilter.create(parent, SWT.FILL);
			break;
		case RANK:
			rankSourceFilter.create(parent, SWT.FILL);
			break;
		case YEAR:
			yearSourceFilter.create(parent, SWT.FILL);
			break;
		case AGE:
			ageSourceFilter.create(parent, SWT.FILL);
			break;
		case SEARCH:
			searchSourceFilter.create(parent, SWT.FILL);
			break;
		default:
			System.out.println("construct " + elem + " (not implemented yet, try another one)");
		}
		
		parent.pack();
		mainComposite.pack();
		mainComposite.layout(true, true);
	}
	
	
	public static Display getDisplay() {
 	   Display display = Display.getCurrent();
 	   // may be null if outside the UI thread
	   if (display == null)
	      display = Display.getDefault();
	   return display;		
    }
	
	

}
